package com.refinitiv.dss.api.example;
/*
 * ====================================================================
 * This Java example demonstrate how to make HTTPS requests to DSS2:
 * - to create an Instrument List
 * - to retrieve an Instrument List by the Instrument List name
 * - to add instruments to the Instrument List
 * - to create a Report Template (in this case a RTH Historical Reference template)
 *   (in this case we use it for cross referencing an instrument)
 * - to retrieve a Report Template by the Report Template name
 * - to create an Immediate Report Schedule  
 * - to check if the Report is complete
 * - to save the extracted files to disk (and also read the saved file from disk)
 * - to retrieve the data stream directly from the server and display its contents
 * - to cleanup: delete created Schedule, Report Template, Instrument List
 * 
 * To run this example, you must provide a DSS2 username and password.
 * ====================================================================
 */

import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.StringWriter;
import java.io.UnsupportedEncodingException;
import java.net.HttpURLConnection;
import java.net.URL;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.time.Instant;
import java.util.Scanner;
import java.util.zip.GZIPInputStream;

import org.apache.commons.compress.compressors.gzip.GzipCompressorInputStream;
import org.apache.http.HttpResponse;
import org.apache.http.client.methods.HttpDelete;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.impl.client.BasicResponseHandler;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.ResponseHandler;
import org.json.JSONArray;
import org.json.JSONObject;
import org.json.JSONOrderedObject;

public class DSSImmediateScheduleHistoricalReferenceRTH {
    private String urlHost = "https://hosted.datascopeapi.reuters.com/RestApi/v1";
    private String sessionToken = "";
    private CloseableHttpClient httpclient = HttpClientBuilder.create().disableContentCompression().build();

    public static void main(String[] args) throws Exception {

        if(args.length < 2) {
            System.out.println("Please enter DSS2 Username as 1st command line parameter, Password as 2nd");
            System.exit(-1);
        }

        DSSImmediateScheduleHistoricalReferenceRTH dss2 = new DSSImmediateScheduleHistoricalReferenceRTH();
        dss2.getSessionToken(args[0], args[1]);
        System.out.println("");

        //* Create Instrument List with a given name
        String listId = dss2.createInstrumentList("Demo_HistoricalReference_Instrumentslist");
        promptEnterKey();

        //* Retrieve Instrument List ID from name (for the case where it was created manually)
        String myListId = dss2.getInstrumentListId("Demo_HistoricalReference_Instrumentslist");
        System.out.println("Instrument List ID received: " +myListId);
        promptEnterKey();

        //* Add hard coded list of instruments to the list id
        dss2.addInstrumentsToList(listId);
        promptEnterKey();

        //* Create the Report Template
        String reportTemplateId = dss2.createRthHistoricalReferenceReportTemplate("Demo_HistoricalReference_Template");
        promptEnterKey();

        //* Retrieve Report Template ID from name (for the case where it was created manually)
        String myReportTemplateId = dss2.getReportTemplateId("Demo_HistoricalReference_Template");
        System.out.println("Report Template ID received: " +myReportTemplateId);
        promptEnterKey();

        //* Create the Immediate Schedule 
        String reportScheduleId = dss2.createReportSchedule("Demo_HistoricalReference_Schedule", listId, reportTemplateId);
        promptEnterKey();

        //* Check if Report is complete
        String reportExtractionId = dss2.checkExtractionComplete(reportScheduleId);
        promptEnterKey();

        //* Display and save extracted files
        dss2.getReportExtractionsFiles(reportExtractionId);
        promptEnterKey();

        //** CLEAN UP **
        //* Delete report Schedule with this id
        dss2.deleteSchedule(reportScheduleId);

        //* Delete the Report Template
        dss2.deleteReportTemplate(reportTemplateId);

        //* Delete Instrument List with a given list id
        dss2.deleteInstrumentList(listId);

        dss2.httpclient.close();
    }


    /**
     * Request DSS2 for a Session Token (24 hour life)
     * 
     * @param username
     * @param password
     */
    public void getSessionToken(String username, String password) {

        try {
            HttpPost httppost = new HttpPost(urlHost + "/Authentication/RequestToken");

            httppost.addHeader("content-type", "application/json; charset=UTF-8");

            JSONObject TokenRequest = new JSONObject()
                .put("Credentials", new JSONObject()
                    .put("Username",  username)
                    .put("Password",  password));

            StringEntity requestBody = new StringEntity(TokenRequest.toString());

            httppost.setEntity(requestBody);

            ResponseHandler<String> responseHandler = new BasicResponseHandler();

            String response = httpclient.execute(httppost, responseHandler);
            JSONObject jsonResponse = new JSONObject(response);

            sessionToken = jsonResponse.get("value").toString();
            System.out.println("Session Token (expires in 24 hours):\n" + sessionToken);

        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }


    /**
     * Create an Instrument List
     * 
     * @param listName
     * @return list ID
     */
    public String createInstrumentList(String listName) {

        String listId = "";
        try {
            HttpPost httppost = new HttpPost(urlHost + "/Extractions/InstrumentLists");

            httppost.addHeader("content-type", "application/json;odata.metadata=minimal");
            httppost.addHeader("Authorization", "Token " + sessionToken);

            JSONOrderedObject instListJSONObject = new JSONOrderedObject()
                .put("@odata.type", "#ThomsonReuters.Dss.Api.Extractions.SubjectLists.InstrumentList")
                .put("Name", listName);

            System.out.println("Instrument list creation JSON request content:\n"+ instListJSONObject.toString());
            StringEntity requestBody = new StringEntity(instListJSONObject.toString());

            httppost.setEntity(requestBody);

            ResponseHandler<String> responseHandler = new BasicResponseHandler();

            String response = httpclient.execute(httppost, responseHandler);
            JSONObject jsonResponse = new JSONObject(response);

            StringWriter out = new StringWriter();
            jsonResponse.write(out);
            System.out.println("JSON response contents:\n"+ out);

            listId = jsonResponse.getString("ListId");

        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }

        return listId;
    }


    /**
     * Get Instrument List ID from name
     * 
     * @param listName
     * @return instrumentListId
     */
    public String getInstrumentListId(String listName) {
    	String myListId = "";
        try {
            String urlGet = urlHost + "/Extractions/InstrumentListGetByName(ListName='" + listName + "')";
            HttpGet request = new HttpGet(urlGet);
            request.addHeader("Authorization", "Token "+sessionToken);

            System.out.println("Retrieve Instrument List with this name: "+ listName);
            HttpResponse response = httpclient.execute(request);

            // It will receive response code 404 if no list is found for that name.
            int httpStatus = response.getStatusLine().getStatusCode();
            System.out.println("HTTP status: " + httpStatus);

            BufferedReader rd = new BufferedReader(
                    new InputStreamReader(response.getEntity().getContent()));

            StringBuffer result = new StringBuffer();
            String line = "";
            while ((line = rd.readLine()) != null) {
                result.append(line);
            }

            System.out.println("JSON response contents:\n"+result);

            JSONObject jsonGetResponse = new JSONObject(result.toString());

            if (httpStatus == 200) { myListId = jsonGetResponse.get("ListId").toString(); };

        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }

        return myListId;
    }


    /**
     * Add Instruments to an Instrument List
     * 
     * @param listId
     */
    public void addInstrumentsToList(String listId) {

        try {
            HttpPost httppost = new HttpPost(urlHost + "/Extractions/InstrumentLists('"+listId+"')/ThomsonReuters.Dss.Api.Extractions.InstrumentListAppendIdentifiers");

            httppost.addHeader("content-type", "application/json;odata.metadata=minimal");
            httppost.addHeader("Authorization", "Token " + sessionToken);

            JSONOrderedObject instListJSONObject = new JSONOrderedObject()
               .put("Identifiers", new JSONArray()
                   .put(new JSONObject()
                       .put("Identifier", "CARR.PA")
                       .put("IdentifierType", "Ric")
                       .put("UserDefinedIdentifier", "RICTEST"))
                   .put(new JSONObject()
                       .put("Identifier", "00209TAB1")
                       .put("IdentifierType", "Cusip")
                       .put("UserDefinedIdentifier", "CUSIPTEST"))
                   .put(new JSONObject()
                       .put("Identifier", "US4592001014")
                       .put("IdentifierType", "Isin")
                       .put("UserDefinedIdentifier", "ISINTEST"))
                   .put(new JSONObject()
                       .put("Identifier", "B1YW440")
                       .put("IdentifierType", "Sedol")
                       .put("UserDefinedIdentifier", "SEDOLTEST"))
                   .put(new JSONObject()
                       .put("Identifier", "INVALID")
                       .put("IdentifierType", "Ric")
                       .put("UserDefinedIdentifier", "INVALID")))
               .put("KeepDuplicates", false);

            System.out.println("Instrument addition JSON request content:\n"+ instListJSONObject.toString());
            StringEntity requestBody = new StringEntity(instListJSONObject.toString());

            httppost.setEntity(requestBody);

            ResponseHandler<String> responseHandler = new BasicResponseHandler();

            String response = httpclient.execute(httppost, responseHandler);
            JSONObject jsonResponse = new JSONObject(response);

            StringWriter out = new StringWriter();
            jsonResponse.write(out);
            System.out.println("JSON response contents:\n"+ out);

        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }


    /**
     * Get all Instruments for the specific Instrument List
     * 
     * @param instListName
     */
    public void getAllInstruments(String listID) {

        try {
            String urlGet = urlHost + "/Extractions/InstrumentLists('" + listID + "')/ThomsonReuters.Dss.Api.Extractions.InstrumentListGetAllInstruments";

            HttpGet request = new HttpGet(urlGet);
            request.addHeader("Authorization", "Token "+sessionToken);

            HttpResponse response = httpclient.execute(request);

            System.out.println("Get all instruments. HTTP status: " 
                + response.getStatusLine().getStatusCode());

            BufferedReader rd = new BufferedReader(
                new InputStreamReader(response.getEntity().getContent()));

            StringBuffer result = new StringBuffer();
            String line = "";
            while ((line = rd.readLine()) != null) {
                result.append(line);
            }

            System.out.println("JSON results received:\n"+result);

            JSONObject jsonGetResponse = new JSONObject(result.toString());

            String userDetail = jsonGetResponse.get("value").toString();
            System.out.println("All Instruments: " +userDetail);

        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }


    /**
     * Delete an Instrument List with this Instrument List ID
     * 
     * @param listID
     */
    public void deleteInstrumentList(String listId) {
        try {

            String urlDel = urlHost + "/Extractions/InstrumentLists('" + listId + "')";

            HttpDelete request = new HttpDelete(urlDel);
            request.addHeader("Authorization", "Token "+sessionToken);

            System.out.println("Delete Instrument List with this id: "+ listId);
            HttpResponse response = httpclient.execute(request);

            System.out.println("HTTP status: " 
                + response.getStatusLine().getStatusCode());

            // It will receive response code 204 with no content
            if (response.getStatusLine().getStatusCode() == 204){
                System.out.println("Delete Instrument List ID " + listId + " completed.");
            }

        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }


    /**
     * Create a RTH Historical Reference Report Template
     * (here we use it for cross referencing an instrument)
     * 
     * @param reportTemplateName
     * @return reportTemplateId
     */
    public String createRthHistoricalReferenceReportTemplate(String reportTemplateName) {

        String reportTemplateId = "";

        try {
            HttpPost httppost = new HttpPost(urlHost + "/Extractions/HistoricalReferenceReportTemplates");

            httppost.addHeader("content-type", "application/json;odata.metadata=minimal");
            httppost.addHeader("Authorization", "Token " + sessionToken);

             // Note: content fields are specific to each report template type
            JSONOrderedObject BarTemplateJSONObject = new JSONOrderedObject()           
                .put("@odata.type", "#ThomsonReuters.Dss.Api.Extractions.ReportTemplates.HistoricalReferenceReportTemplate")
                .put("ShowColumnHeaders", false)
                .put("Name", reportTemplateName)
                .put("Headers", new JSONArray() )
                .put("Trailers", new JSONArray() )
                .put("ContentFields", new JSONArray()
                    .put( new JSONObject()
                        .put("FieldName", "RIC")
                        .put( "Format", JSONObject.NULL))
                    .put( new JSONObject()
                        .put("FieldName", "CUSIP")
                        .put( "Format", JSONObject.NULL))
                    .put( new JSONObject()
                        .put("FieldName", "ISIN")
                        .put( "Format", JSONObject.NULL) )
                    .put( new JSONObject()
                        .put("FieldName", "SEDOL")
                        .put( "Format", JSONObject.NULL) )
                    .put( new JSONObject()
                        .put("FieldName", "Issuer OrgID")
                        .put( "Format", JSONObject.NULL) )
                    .put( new JSONObject()
                        .put("FieldName", "Exchange Code")
                        .put( "Format", JSONObject.NULL) )
                    .put( new JSONObject()
                        .put("FieldName", "Currency Code")
                        .put( "Format", JSONObject.NULL) )
                    .put( new JSONObject()
                        .put("FieldName", "Change Date")
                        .put( "Format", JSONObject.NULL) ) )
                .put("Condition", new JSONOrderedObject()
                	.put( "ReportDateRangeType", "Range" )
                	.put( "QueryStartDate", "2017-05-31" )
                    .put( "QueryEndDate", "2017-05-31" ));

            System.out.println("Historical reference template creation JSON request content:\n"+ BarTemplateJSONObject.toString());
            StringEntity requestBody = new StringEntity(BarTemplateJSONObject.toString());

            httppost.setEntity(requestBody);

            ResponseHandler<String> responseHandler = new BasicResponseHandler();

            String response = httpclient.execute(httppost, responseHandler);
            JSONObject jsonResponse = new JSONObject(response);

            StringWriter out = new StringWriter();
            jsonResponse.write(out);
            System.out.println("JSON response contents:\n"+ out);
            reportTemplateId = jsonResponse.getString("ReportTemplateId");

        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }

        return reportTemplateId;
    }


    /**
     * Get Report Template ID from name
     * 
     * @param reportTemplateName
     * @return reportTemplateId
     */
    public String getReportTemplateId(String reportTemplateName) {
    	String myReportTemplateId = "";
        try {
            String urlGet = urlHost + "/Extractions/ReportTemplateGetByName(Name='" + reportTemplateName + "')";
            HttpGet request = new HttpGet(urlGet);
            request.addHeader("Authorization", "Token "+sessionToken);

            System.out.println("Retrieve Report Template with this name: "+ reportTemplateName);
            HttpResponse response = httpclient.execute(request);

            // It will receive response code 404 if no template is found for that name.
            int httpStatus = response.getStatusLine().getStatusCode();
            System.out.println("HTTP status: " + httpStatus);

            BufferedReader rd = new BufferedReader(
                new InputStreamReader(response.getEntity().getContent()));

            StringBuffer result = new StringBuffer();
            String line = "";
            while ((line = rd.readLine()) != null) {
                result.append(line);
            }

            System.out.println("JSON response contents:\n"+result);

            JSONObject jsonGetResponse = new JSONObject(result.toString());

            if (httpStatus == 200) { myReportTemplateId = jsonGetResponse.get("ReportTemplateId").toString(); };

        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }

        return myReportTemplateId;
    }


    /**
     * Get Report Template for this reportTemplateId
     * 
     * @param reportTemplateId
     */
    public void getReportTemplate(String reportTemplateId) {
        try {
            String urlGet = urlHost + "/Extractions/ReportTemplates('"+reportTemplateId+"')";
            HttpGet request = new HttpGet(urlGet);
            request.addHeader("Authorization", "Token "+sessionToken);

            System.out.println("Retrieve Report Template with this Id: "+ reportTemplateId);
            HttpResponse response = httpclient.execute(request);

            // It will receive response code 404 if no template is found for that id.
            System.out.println("HTTP status: " 
                + response.getStatusLine().getStatusCode());

            BufferedReader rd = new BufferedReader(
                new InputStreamReader(response.getEntity().getContent()));

            StringBuffer result = new StringBuffer();
            String line = "";
            while ((line = rd.readLine()) != null) {
                result.append(line);
            }

            System.out.println("Report Template received:\n"+result);

        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }


    /**
     * Get all Report Templates
     * 
     */
    public void getAllReportTemplates() {
        try {
            String urlGet = urlHost + "/Extractions/ReportTemplates";
            HttpGet request = new HttpGet(urlGet);
            request.addHeader("Authorization", "Token "+sessionToken);

            HttpResponse response = httpclient.execute(request);

            System.out.println("Get all report templates. HTTP status: " 
                + response.getStatusLine().getStatusCode());

            BufferedReader rd = new BufferedReader(
                new InputStreamReader(response.getEntity().getContent()));

            StringBuffer result = new StringBuffer();
            String line = "";
            while ((line = rd.readLine()) != null) {
                result.append(line);
            }

            System.out.println("Report Templates received:\n"+result);

            JSONObject jsonGetResponse = new JSONObject(result.toString());

            String userDetail = jsonGetResponse.get("value").toString(); 
            System.out.println("All Report Templates:\n" +userDetail);

        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }


    /**
     * Delete Report Template with this reportTemplateId
     * 
     * @param reportTemplateId
     */
    public void deleteReportTemplate(String reportTemplateId) {
        try {

            String urlDel = urlHost + "/Extractions/ReportTemplates('"+reportTemplateId+"')";

            HttpDelete request = new HttpDelete(urlDel);
            request.addHeader("Authorization", "Token "+sessionToken);

            System.out.println("Delete Report Template with this id: "+ reportTemplateId);
            HttpResponse response = httpclient.execute(request);

            System.out.println("HTTP status: " 
                + response.getStatusLine().getStatusCode());

            // It will receive response code 204 with no content
            if (response.getStatusLine().getStatusCode() == 204){
                System.out.println("Delete Report Template ID " + reportTemplateId + " completed.");
            }

        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }


    /**
     * Create an Immediate Report Schedule
     * 
     * @param reportScheduleName
     * @param listId
     * @param reportTemplateId
     * @return reportScheduleId
     */
    public String createReportSchedule(String reportScheduleName, String listId, String reportTemplateId) {

        String reportScheduleId = "";

        try {
            HttpPost httppost = new HttpPost(urlHost + "/Extractions/Schedules"); 

            httppost.addHeader("content-type", "application/json;odata.metadata=minimal");
            httppost.addHeader("Authorization", "Token " + sessionToken);

            // Java 8 +
            Instant instant = Instant.now ();  // Current date-time in UTC.
            String currDatetime = instant.toString ();

            JSONOrderedObject ScheduleJSONObject = new JSONOrderedObject()
                .put("TimeZone", "Eastern Standard Time")
                .put("Name", reportScheduleName)  
                .put("Recurrence", new JSONOrderedObject()
                    .put("@odata.type", "#ThomsonReuters.Dss.Api.Extractions.Schedules.SingleRecurrence")
                    .put("ExtractionDateTime", currDatetime)
                    .put("IsImmediate", true))
                .put("Trigger", new JSONOrderedObject()
                    .put("@odata.type", "#ThomsonReuters.Dss.Api.Extractions.Schedules.ImmediateTrigger")
                    .put("LimitReportToTodaysData", false))
                .put("ListId", listId)
                .put("ReportTemplateId", reportTemplateId);

            System.out.println("Schedule creation JSON request content:\n"+ ScheduleJSONObject.toString());
            StringEntity requestBody = new StringEntity(ScheduleJSONObject.toString());

            httppost.setEntity(requestBody);

            ResponseHandler<String> responseHandler = new BasicResponseHandler();

            String response = httpclient.execute(httppost, responseHandler);
            JSONObject jsonResponse = new JSONObject(response);

            StringWriter out = new StringWriter();
            jsonResponse.write(out);
            System.out.println("JSON response contents:\n"+ out);
            reportScheduleId = jsonResponse.getString("ScheduleId");

        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }

        return reportScheduleId;
    }


    /**
     * Delete Schedule with this schedule Id
     * 
     * @param ScheduleId
     */
    public void deleteSchedule(String ScheduleId) {
        try {

            String urlDel = urlHost + "/Extractions/Schedules('"+ScheduleId+"')";

            HttpDelete request = new HttpDelete(urlDel);
            request.addHeader("Authorization", "Token "+sessionToken);

            System.out.println("Delete Schedule with this id: "+ ScheduleId);
            HttpResponse response = httpclient.execute(request);

            System.out.println("HTTP status: " 
                    + response.getStatusLine().getStatusCode());

            // It will receive response code 204 with no content
            if (response.getStatusLine().getStatusCode() == 204){
                System.out.println("Delete Schedule ID " + ScheduleId + " completed.");
            }

        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }   


    /**
     * Check the Status of the Immediate Schedule request, wait for it to complete
     * 
     * @param reportScheduleId
     * @return reportExtractionId
     */
    public String checkExtractionComplete(String reportScheduleId) {

        String reportExtractionId = "";

        try {
            String urlGet = urlHost + "/Extractions/Schedules('"+reportScheduleId+"')/LastExtraction";

            HttpGet request = new HttpGet(urlGet);
            request.addHeader("Authorization", "Token "+sessionToken);

            HttpResponse response = httpclient.execute(request);

            // Get the response status code
            int respStatusCode = response.getStatusLine().getStatusCode();
            System.out.println("Check extraction complete. HTTP status: " + respStatusCode);

            // If the status code is 200, then a response is available now.
            // Otherwise, poll (best practice: interval >= 30 sec)
            while (respStatusCode == 204) {
                wait(30);
                response = httpclient.execute(request);
                respStatusCode = response.getStatusLine().getStatusCode();

                System.out.println("HTTP status: " + respStatusCode);
            }

            switch (respStatusCode) {
                case 200:    // if the status code is 200, then a response is available now
                    BufferedReader rd = new BufferedReader(
                        new InputStreamReader(response.getEntity().getContent()));

                    StringBuffer result = new StringBuffer();
                    String line = "";
                    while ((line = rd.readLine()) != null) {
                        result.append(line);
                    }

                    System.out.println("Results received:\n"+result);

                    JSONObject jsonGetResponse = new JSONObject(result.toString());

                    String extractionStatus = jsonGetResponse.get("Status").toString();
                    System.out.println("Extraction ready? : " + extractionStatus);

                    reportExtractionId = jsonGetResponse.get("ReportExtractionId").toString();
                    System.out.println("ReportExtractionId: " + reportExtractionId);
                    break;

                case 400:
                    System.out.println("ERROR: HTTP status: 400 (Bad Request).  Request content is malformed and cannot be parsed.");
                    break;
                case 401:
                    System.out.println("ERROR: HTTP status: 401 (Unauthorized).  Authentication token is invalid or has expired.");
                    break;
                case 403:
                    System.out.println("ERROR: HTTP status: 403 (Forbidden).  Account not permissioned for this type of data.");
                    break;
                default:
                    System.out.println("ERROR: Cannot proceed. Please check the meaning of HTTP status " + respStatusCode);
                    break;
            }

        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }

        return reportExtractionId;
    }


    /**
     * Get the Report Extraction Files
     * 
     * @param ReportExtractionId
     */
    public void getReportExtractionsFiles(String reportExtractionId) {

        try {
            String urlGet = urlHost + "/Extractions/ReportExtractions('"+reportExtractionId+"')/Files";

            HttpGet request = new HttpGet(urlGet);
            request.addHeader("Authorization", "Token "+sessionToken);

            HttpResponse response = httpclient.execute(request);

            System.out.println("Get report extraction files. HTTP status: " 
                    + response.getStatusLine().getStatusCode());

            BufferedReader rd = new BufferedReader(
                    new InputStreamReader(response.getEntity().getContent()));

            StringBuffer result = new StringBuffer();
            String line = "";
            while ((line = rd.readLine()) != null) {
                result.append(line);
            }

            System.out.println("JSON results received:\n"+result);

            JSONObject jsonGetResponse = new JSONObject(result.toString());

            JSONArray valueJArray = jsonGetResponse.getJSONArray("value");
            System.out.println("\nFiles to extract:\n" + valueJArray);

            String ExtractedFileId = "";
            String ExtractedFileName = "";
            String FileType = "";
            int FileNumber = 0;
            int FileSize = 0;
            boolean FileIsCompressed = false;
            // There will be several files, typically a data file and a Notes file.
            // Depending on the preferences setting there might also be a maintenance file
            for (int i = 0; i < valueJArray.length(); i++)
            {
            	FileNumber++;
                System.out.println("\n=============\n" + "File number " + FileNumber + "\n=============");
                ExtractedFileId = valueJArray.getJSONObject(i).getString("ExtractedFileId");
                System.out.println("\nExtractedFileId("+i+"): "+ ExtractedFileId);
                System.out.println("ReportExtractionId("+i+"): "+ valueJArray.getJSONObject(i).getString("ReportExtractionId"));
                //The schedule ID is not returned by a Historical Reference call
                //System.out.println("ScheduleId("+i+"): "+ valueJArray.getJSONObject(i).getString("ScheduleId"));
                FileType = valueJArray.getJSONObject(i).getString("FileType");
                System.out.println("FileType("+i+"): "+ FileType);

                ExtractedFileName = valueJArray.getJSONObject(i).getString("ExtractedFileName");
                System.out.println("ExtractedFileName("+i+"): "+ ExtractedFileName);

                System.out.println("LastWriteTimeUtc("+i+"): "+ valueJArray.getJSONObject(i).getString("LastWriteTimeUtc"));
                System.out.println("ContentsExists("+i+"): "+ valueJArray.getJSONObject(i).getBoolean("ContentsExists"));
                FileSize = valueJArray.getJSONObject(i).getInt("Size");
                System.out.println("Size("+i+"): "+ FileSize);

                if (FileSize>0) {
                    //Get the file and save it to local disk
                    extractAndSaveFile(ExtractedFileName, ExtractedFileId);

                    //As an example we shall also display the contents of the files
                    FileIsCompressed = ExtractedFileName.endsWith(".gz");
                	if (FileIsCompressed)
                	{
                        //Read compressed file back from disk
                        readFileAndDecompress(ExtractedFileName);
                        //Alternative: display file contents directly (without local save)
                        extractDataDecompressAndDisplayContents(ExtractedFileId);               		
                	} else {
                		extractDataAndDisplayContents(ExtractedFileId);
                	}
                }
            }

        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }


    /**
     * Extract a report file, save it to local disk
     * 
     * @param filename
     * @param ExtractedFileId
     */
    public void extractAndSaveFile( String filename, String ExtractedFileId) {

        System.out.println("\nRetrieve file from server, and save directly to hard disk:\n "+ filename);

        String urlGet = urlHost + "/Extractions/ExtractedFiles('"+ExtractedFileId+"')/$value";

        try {
            URL myURL = new URL(urlGet);

            HttpURLConnection myURLConnection = (HttpURLConnection)myURL.openConnection();

            myURLConnection.setRequestProperty("Authorization", "Token "+sessionToken);
            myURLConnection.setRequestProperty("Accept-Charset", "UTF-8");
            myURLConnection.setRequestMethod("GET");

            //This works both with compressed and non compressed files
            try( DataInputStream readerIS = new DataInputStream( myURLConnection.getInputStream())) {
                Files.copy (readerIS, Paths.get (filename));
            }
            System.out.println("Saved file to hard disk.");

        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }


    /**
     * Read a compressed file from local disk, decompress contents and display
     * 
     * @param filename
     */
    public void readFileAndDecompress( String filename) {

    	System.out.println("\nRead compressed file from hard disk, and decompress contents.");

        try (
            InputStream theFile = new FileInputStream(filename);
            GZIPInputStream gis = new GZIPInputStream(theFile);
        ) {
            StringBuffer result = new StringBuffer();
            String line = "";
            long lineCount = 0;
            BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(gis, "UTF-8"));
            while ((line = bufferedReader.readLine()) != null) {
                ++lineCount;
                line = line + "\n";
                result.append(line);
            }
            System.out.println("\nGzipped File contents:\n"+result+"\nLine count: "+lineCount);

        } catch (IOException e) {
            e.printStackTrace();
        }
    }


    /**
     * Extract a compressed file streamed from the server, decompress it, and simply display its contents
     *  
     * @param ExtractedFileId
     */
    public void extractDataDecompressAndDisplayContents(String ExtractedFileId) {

        System.out.println("\nRetrieve compressed data directly from server, and decompress contents on the fly.");

        String urlGet = urlHost + "/Extractions/ExtractedFiles('"+ExtractedFileId+"')/$value";

        try {
            URL myURL = new URL(urlGet);

            HttpURLConnection myURLConnection = (HttpURLConnection)myURL.openConnection();

            myURLConnection.setRequestProperty("Authorization", "Token "+sessionToken);
            myURLConnection.setRequestProperty("Accept-Charset", "UTF-8");
            myURLConnection.setRequestMethod("GET");

            try (
                GzipCompressorInputStream gis = new GzipCompressorInputStream( myURLConnection.getInputStream(), true);
                BufferedReader rd = new BufferedReader(new InputStreamReader(gis, "UTF-8"));
            ) {
                StringBuffer result = new StringBuffer();
                String line = "";
                long lineCount = 0;
                while ((line = rd.readLine()) != null) {
                    ++lineCount;
                    line = line + "\n";
                    result.append(line);
                }
                System.out.println("\nData file contents:\n"+result+"\nLine count: "+lineCount);
            }

        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }


    /**
     * Extract a non compressed file streamed from the server, and simply display its contents
     *  
     * @param ExtractedFileId
     */
    public void extractDataAndDisplayContents(String ExtractedFileId) {

        String urlGet = urlHost + "/Extractions/ExtractedFiles('"+ExtractedFileId+"')/$value";

        try {
            HttpGet request = new HttpGet(urlGet);
            request.addHeader("Authorization", "Token "+sessionToken);

            HttpResponse response = httpclient.execute(request);

            System.out.println("\nGet uncompressed file directly from server. HTTP status: " 
                    + response.getStatusLine().getStatusCode());

            BufferedReader rd = new BufferedReader(
                    new InputStreamReader(response.getEntity().getContent()));

            StringBuffer result = new StringBuffer();
            String line = "";
            while ((line = rd.readLine()) != null) {
            	line = line + "\n";
            	result.append(line);
            }
            System.out.println("\nFile contents:\n"+result);
            rd.close();

        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }


    public static void wait(int seconds) {
        try {
            System.out.println("Waiting " + seconds + " seconds ..");
            Thread.sleep(seconds * 1000);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }        
    }


    public static void promptEnterKey(){
        System.out.println("Press \"ENTER\" to continue...");
        Scanner scanner = new Scanner(System.in);
        scanner.nextLine();
    }    
} //EOF
