package com.refinitiv.dss.api.example;
/*
 * ====================================================================
 * This Java example demonstrate how to make an HTTPS request to DSS2:
 * - to create an Instrument List
 * - to retrieve an Instrument List by the Instrument List name
 * - to add instruments to the Instrument List
 * - to query for Instruments in an Instrument List
 * - to delete an Instrument List,
 * 
 * To run this example, you must provide a DSS2 username and password.
 * ====================================================================
 */

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.StringWriter;
import java.io.UnsupportedEncodingException;
import java.util.Scanner;

import org.apache.http.HttpResponse;
import org.apache.http.client.methods.HttpDelete;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.impl.client.BasicResponseHandler;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.ResponseHandler;
import org.json.JSONArray;
import org.json.JSONObject;
import org.json.JSONOrderedObject;

public class DSSInstruments {
    private String urlHost = "https://hosted.datascopeapi.reuters.com/RestApi/v1";
    private String sessionToken = "";
    private CloseableHttpClient httpclient = HttpClientBuilder.create().disableContentCompression().build();

    public static void main(String[] args) throws Exception {

        if(args.length < 2) {
            System.out.println("Please enter DSS2 Username as 1st command line parameter, Password as 2nd");
            System.exit(-1);
        }

        DSSInstruments dss2 = new DSSInstruments();
        dss2.getSessionToken(args[0], args[1]);
        System.out.println("");

        //* Create Instrument List with a given name
        String listId = dss2.createInstrumentList("Demo_Instrumentslist");
        promptEnterKey();

        //* Retrieve Instrument List ID from name (for the case where it was created manually)
        String myListId = dss2.getInstrumentListId("Demo_Instrumentslist");
        System.out.println("Instrument List ID received: " +myListId);
        promptEnterKey();

        //* Add hard coded list of instruments to the list id
        dss2.addInstrumentsToList(listId);
        promptEnterKey();

        // Retrieve all instruments from list id
        dss2.getAllInstruments(listId);
        promptEnterKey();

        // Delete Instrument List with a given list id
        dss2.deleteInstrumentList(listId);

        dss2.httpclient.close();
    }


    /**
     * Request DSS2 for a Session Token (24 hour life)
     * 
     * @param username
     * @param password
     */
    public void getSessionToken(String username, String password) {

        try {
            HttpPost httppost = new HttpPost(urlHost + "/Authentication/RequestToken");

            httppost.addHeader("content-type", "application/json; charset=UTF-8");

            JSONObject TokenRequest = new JSONObject()
                .put("Credentials", new JSONObject()
                    .put("Username",  username)
                    .put("Password",  password));

            StringEntity requestBody = new StringEntity(TokenRequest.toString());

            httppost.setEntity(requestBody);

            ResponseHandler<String> responseHandler = new BasicResponseHandler();

            String response = httpclient.execute(httppost, responseHandler);
            JSONObject jsonResponse = new JSONObject(response);

            sessionToken = jsonResponse.get("value").toString();
            System.out.println("Session Token (expires in 24 hours):\n" + sessionToken);

        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }


    /**
     * Create an Instrument List
     * 
     * @param listName
     * @return listId
     */
    public String createInstrumentList(String listName) {

        String listId = "";
        try {
            HttpPost httppost = new HttpPost(urlHost + "/Extractions/InstrumentLists");

            httppost.addHeader("content-type", "application/json;odata.metadata=minimal");
            httppost.addHeader("Authorization", "Token " + sessionToken);

            JSONOrderedObject instListJSONObject = new JSONOrderedObject()
                .put("@odata.type", "#ThomsonReuters.Dss.Api.Extractions.SubjectLists.InstrumentList")
                .put("Name", listName);

            System.out.println("Instrument list creation JSON request content:\n"+ instListJSONObject.toString());
            StringEntity requestBody = new StringEntity(instListJSONObject.toString());

            httppost.setEntity(requestBody);

            ResponseHandler<String> responseHandler = new BasicResponseHandler();

            String response = httpclient.execute(httppost, responseHandler);
            JSONObject jsonResponse = new JSONObject(response);

            StringWriter out = new StringWriter();
            jsonResponse.write(out);
            System.out.println("JSON response contents:\n"+ out);

            listId = jsonResponse.getString("ListId");

        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }

        return listId;
    }


    /**
     * Get Instrument List ID from name
     * 
     * @param listName
     * @return instrumentListId
     */
    public String getInstrumentListId(String listName) {
    	String myListId = "";
        try {
            String urlGet = urlHost + "/Extractions/InstrumentListGetByName(ListName='" + listName + "')";
            HttpGet request = new HttpGet(urlGet);
            request.addHeader("Authorization", "Token "+sessionToken);

            System.out.println("Retrieve Instrument List with this name: "+ listName);
            HttpResponse response = httpclient.execute(request);

            // It will receive response code 404 if no list is found for that name.
            int httpStatus = response.getStatusLine().getStatusCode();
            System.out.println("HTTP status: " + httpStatus);

            BufferedReader rd = new BufferedReader(
                    new InputStreamReader(response.getEntity().getContent()));

            StringBuffer result = new StringBuffer();
            String line = "";
            while ((line = rd.readLine()) != null) {
                result.append(line);
            }

            System.out.println("JSON response contents:\n"+result);

            JSONObject jsonGetResponse = new JSONObject(result.toString());

            if (httpStatus == 200) { myListId = jsonGetResponse.get("ListId").toString(); };

        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }

        return myListId;
    }


    /**
     * Add Instruments to an Instrument List
     * 
     * @param listId
     */
    public void addInstrumentsToList(String listId) {

        try {
            HttpPost httppost = new HttpPost(urlHost + "/Extractions/InstrumentLists('"+listId+"')/ThomsonReuters.Dss.Api.Extractions.InstrumentListAppendIdentifiers");

            httppost.addHeader("content-type", "application/json;odata.metadata=minimal");
            httppost.addHeader("Authorization", "Token " + sessionToken);

            JSONOrderedObject instListJSONObject = new JSONOrderedObject()
               .put("Identifiers", new JSONArray()
                   .put(new JSONObject()
                        .put("Identifier", "ONCE.O")
                        .put("IdentifierType", "Ric")
                        .put("UserDefinedIdentifier", "OptionalUserIdentifier")))
               .put("KeepDuplicates", false);

            System.out.println("Instrument addition JSON request content:\n"+ instListJSONObject.toString());
            StringEntity requestBody = new StringEntity(instListJSONObject.toString());

            httppost.setEntity(requestBody);

            ResponseHandler<String> responseHandler = new BasicResponseHandler();

            String response = httpclient.execute(httppost, responseHandler);
            JSONObject jsonResponse = new JSONObject(response);

            StringWriter out = new StringWriter();
            jsonResponse.write(out);
            System.out.println("JSON response contents:\n"+ out);

        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }


    /**
     * Get all Instruments for the specific Instrument List
     * 
     * @param instListName
     */
    public void getAllInstruments(String listID) {

        try {
            String urlGet = urlHost + "/Extractions/InstrumentLists('" + listID + "')/ThomsonReuters.Dss.Api.Extractions.InstrumentListGetAllInstruments";

            HttpGet request = new HttpGet(urlGet);
            request.addHeader("Authorization", "Token "+sessionToken);

            HttpResponse response = httpclient.execute(request);

            System.out.println("Get all instruments. HTTP status: " 
                + response.getStatusLine().getStatusCode());

            BufferedReader rd = new BufferedReader(
                new InputStreamReader(response.getEntity().getContent()));

            StringBuffer result = new StringBuffer();
            String line = "";
            while ((line = rd.readLine()) != null) {
                result.append(line);
            }

            System.out.println("JSON results received:\n"+result);

            JSONObject jsonGetResponse = new JSONObject(result.toString());

            String userDetail = jsonGetResponse.get("value").toString();
            System.out.println("All Instruments: " +userDetail);

        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }


    /**
     * Delete an Instrument List with this Instrument List ID
     * 
     * @param listID
     */
    public void deleteInstrumentList(String listId) {
        try {

            String urlDel = urlHost + "/Extractions/InstrumentLists('" + listId + "')";

            HttpDelete request = new HttpDelete(urlDel);
            request.addHeader("Authorization", "Token "+sessionToken);

            System.out.println("Delete Instrument List with this id: "+ listId);
            HttpResponse response = httpclient.execute(request);

            System.out.println("HTTP status: " 
                + response.getStatusLine().getStatusCode());

            // It will receive response code 204 with no content
            if (response.getStatusLine().getStatusCode() == 204){
                System.out.println("Delete Instrument List ID " + listId + " completed.");
            }

        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }


    public static void promptEnterKey(){
        System.out.println("Press \"ENTER\" to continue...");
        Scanner scanner = new Scanner(System.in);
        scanner.nextLine();
    }    
} //EOF
