package com.refinitiv.dss.api.example;
/*
 * ====================================================================
 * This Java example demonstrate how to make an HTTPS request to DSS2
 * for a historical Ticks On Demand Extraction report.
 * 
 * This sample can download from RTH or AWS (Amazon Web Services).
 * Note: AWS is only available for VBD files and the following extractions:
 * - Tick History Time and Sales,
 * - Tick History Market Depth,
 * - Tick History Intraday Summaries,
 * - Tick History Raw reports. 
 * 
 * To run this example, you must provide a DSS2 username and password.
 * ====================================================================
 */

import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.StringWriter;
import java.io.UnsupportedEncodingException;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URL;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Scanner;
import java.util.zip.GZIPInputStream;

import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.protocol.HttpClientContext;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.impl.client.BasicResponseHandler;
import org.apache.commons.compress.compressors.gzip.GzipCompressorInputStream;
import org.apache.http.Header;
import org.apache.http.HttpResponse;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.ResponseHandler;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.json.JSONOrderedObject;

public class DSSOnDemandTicksRTH {
    private String urlHost = "https://hosted.datascopeapi.reuters.com/RestApi/v1";
    private String sessionToken = "";
    private CloseableHttpClient httpclient = HttpClientBuilder.create().disableContentCompression().build();
    //Set useAws to:
    //- false to download from the RTH server,
    //- true to use direct download from Amazon Web Services (faster).
    private Boolean useAws = false;

    public static void main(String[] args) throws Exception {

        if(args.length < 2) {
            System.out.println("Please enter DSS2 Username as 1st command line parameter, Password as 2nd");
            System.exit(-1);
        }

        DSSOnDemandTicksRTH dss2 = new DSSOnDemandTicksRTH();
        dss2.getSessionToken(args[0], args[1]);
        System.out.println("");

        // End Of Day On Demand Extraction report
        dss2.ticksExtractRequest();

        dss2.httpclient.close();
    }


    /**
     * Request DSS2 for a Session Token (24 hour life)
     * 
     * @param username
     * @param password
     */
    public void getSessionToken(String username, String password) {

        try {
            HttpPost httppost = new HttpPost(urlHost + "/Authentication/RequestToken");

            httppost.addHeader("content-type", "application/json; charset=UTF-8");

            JSONObject TokenRequest = new JSONObject()
                .put("Credentials", new JSONObject()
                    .put("Username",  username)
                    .put("Password",  password));

            StringEntity requestBody = new StringEntity(TokenRequest.toString());

            httppost.setEntity(requestBody);

            ResponseHandler<String> responseHandler = new BasicResponseHandler();

            String response = httpclient.execute(httppost, responseHandler);
            JSONObject jsonResponse = new JSONObject(response);

            sessionToken = jsonResponse.get("value").toString();
            System.out.println("Session Token (expires in 24 hours):\n" + sessionToken);

        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }


    /**
     * This is a Time and Sales (i.e. ticks) On Demand Extraction Request
     * 
     * Here is the reference doc for the Async mechanism:
     * https://hosted.datascopeapi.reuters.com/RestApi.Help/Home/KeyMechanisms#sectionAsync
     * 
     */
    public void ticksExtractRequest() {

        try {
            HttpPost httppost = new HttpPost(urlHost + "/Extractions/ExtractRaw");

            httppost.addHeader("content-type", "application/json;odata.metadata=minimal");
            httppost.addHeader("Authorization", "Token " + sessionToken);
            // Use the 1st line for your program, the 2nd line to test a timeout (HTTP status 202):
            httppost.addHeader("Prefer", "respond-async");
            //httppost.addHeader("Prefer", "respond-async, wait=5");

            JSONOrderedObject TicksExtractionRequest = new JSONOrderedObject()
                .put("ExtractionRequest", new JSONOrderedObject()
                    .put("@odata.type", "#ThomsonReuters.Dss.Api.Extractions.ExtractionRequests.TickHistoryTimeAndSalesExtractionRequest")
                    .put("ContentFieldNames", new JSONArray()
                    	.put("Trade - Price")
                        .put("Trade - Volume")
                        .put("Trade - Exchange Time"))
                    .put("IdentifierList", new JSONOrderedObject()
                        .put("@odata.type", "#ThomsonReuters.Dss.Api.Extractions.ExtractionRequests.InstrumentIdentifierList")
                        .put("InstrumentIdentifiers", new JSONArray()
                            .put(new JSONObject()
                                .put("Identifier", "438516AC0")
                                .put("IdentifierType", "Cusip"))
                            .put(new JSONObject()
                                .put("Identifier", "CARR.PA")
                                .put("IdentifierType", "Ric"))
                            .put(new JSONObject()
                                .put("Identifier", "INVALID")
                                .put("IdentifierType", "Ric")))
                        .put("ValidationOptions", JSONObject.NULL)
                        .put("UseUserPreferencesForValidationOptions", false))
                    .put("Condition", new JSONOrderedObject()
                    	.put("MessageTimeStampIn", "GmtUtc")
                        .put("ApplyCorrectionsAndCancellations", true )
                    	.put("ReportDateRangeType", "Range")
                        .put("QueryStartDate", "2016-09-29T15:15:00.000Z")
                        .put("QueryEndDate", "2016-09-29T15:16:00.000Z")
                    	.put("DisplaySourceRIC", true)));

            System.out.println("Ticks extraction JSON request content:\n"+ TicksExtractionRequest.toString() +"\n");
            StringEntity requestBody = new StringEntity(TicksExtractionRequest.toString());

            httppost.setEntity(requestBody);

            // NOTE: If the extraction request takes more than 30 seconds the async mechanism will be used.
            System.out.println("We submit the request, a response should arrive in approximately 30 seconds ...\n");
            HttpResponse response = httpclient.execute(httppost);

            StringBuffer result = new StringBuffer();

            // Get the response status code
            int respStatusCode = response.getStatusLine().getStatusCode();
            
            if (respStatusCode==202) {
                // If the request takes longer than the poll time, a status code 202 (accepted)
                // is returned to the client, with a location URL to poll for the status.    
                // The request is accepted, but the result is not yet ready. This is the usual case.
                System.out.println("HTTP status: " + respStatusCode + " - We must wait, and poll the location URL." );

                // Extract the location URL from the response headers:
                Header[] headers = response.getAllHeaders();
                String pollURL = "";
                for (int i= 0; i< headers.length; i++)
                {
                    if (headers[i].getName().equalsIgnoreCase("Location")) {
                        pollURL = headers[i].getValue();
                        System.out.println("Location URL: " + pollURL);
                    }
                }

                // Poll the location URL until the extraction is completed:
                wait(30);

                HttpGet requestGet = new HttpGet(pollURL);
                requestGet.addHeader("Authorization", "Token "+sessionToken);
                requestGet.addHeader("Prefer", "respond-async");

                response = httpclient.execute(requestGet);
                respStatusCode = response.getStatusLine().getStatusCode();
                System.out.println("\nHTTP status: " + respStatusCode);

                while (respStatusCode == 202) {
                    wait(30);
                    response = httpclient.execute(requestGet);
                    respStatusCode = response.getStatusLine().getStatusCode();
                    System.out.println("\nHTTP status: " + respStatusCode);
                }
            }

            if (respStatusCode==200) {
                // A response is available now.
                // Read the response body:
                BufferedReader rd = new BufferedReader(
                    new InputStreamReader(response.getEntity().getContent()));

                String line = "";
                while ((line = rd.readLine()) != null) {
                    result.append(line);
                }
                System.out.println("JSON response contents:\n"+result);

                JSONObject jsonGetResponse = new JSONObject(result.toString());

                // Extract the Job Id from the response body:
                String jobId = jsonGetResponse.get("JobId").toString();
                System.out.println("Job Id: " + jobId);

                // Pull out the extraction notes, check for success:
                JSONArray notes = jsonGetResponse.getJSONArray("Notes");
                StringWriter out = new StringWriter();
                notes.write(out);
                
                System.out.println("\nNotes extracted from JSON response:\n"+ out);
                if (out.toString().length()<1) {
                    System.out.println("WARNING: Notes are empty");
                } else {
                    if (out.toString().contains("Processing completed successfully")) {
                        System.out.println("SUCCESS: Processing completed successfully !");
                    } else {
                        System.out.println("ERROR: Processing did not complete successfully !");
                    }
                }

                // We retrieve the data, using the jobId
                // We can treat it on the fly, and display it
                if (useAws) {
                    awsRetrieveDataDecompressAndDisplayContents(jobId);	
                } else {
                    retrieveDataDecompressAndDisplayContents(jobId);	
                }                        
                        
                // We can also simply save the data to a file on the local disk
                Date date = new Date() ;
                SimpleDateFormat dateFormat = new SimpleDateFormat("ddMMyyyy_HHmmss");
                String Filename = "./Demo_OnDemandTicks_" + dateFormat.format(date) + ".csv.gz";
                if (useAws) {
                    awsRetrieveDataAndSaveToFile(Filename, jobId);	
                } else {
                    retrieveDataAndSaveToFile(Filename, jobId);	
                }                        
                // And read it back later
                readFileAndDecompress(Filename);
            }

            if (respStatusCode!=200) {
            	// If at this stage the latest status code is not 200, we ran into an issue
            	switch (respStatusCode) {
                    case 400:
                        System.out.println("ERROR: HTTP status: 400 (Bad Request).  Request content is malformed and cannot be parsed.");
                        break;
                    case 401:
                        System.out.println("ERROR: HTTP status: 401 (Unauthorized).  Authentication token is invalid or has expired.");
                        break;
                    case 403:
                        System.out.println("ERROR: HTTP status: 403 (Forbidden).  Account not permissioned for this type of data.");
                        break;
                    default:
                        System.out.println("ERROR: Cannot proceed. Please check the meaning of HTTP status " + respStatusCode);
                        break;
                }
            }

        } catch (MalformedURLException e) {
            e.printStackTrace();
        } catch (JSONException e) {
            e.printStackTrace();
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }


    /**
     * Extract a file streamed from the server, save it to local disk
     * 
     * @param filename
     * @param jobId
     */
    public void retrieveDataAndSaveToFile( String filename, String jobId) {

        System.out.println("\nRetrieve data directly from the RTH server, and save to hard disk in file:\n "+ filename);

        String urlGet = urlHost + "/Extractions/RawExtractionResults('"+ jobId + "')/$value";

        try {
            URL myURL = new URL(urlGet);

            HttpURLConnection myURLConnection = (HttpURLConnection)myURL.openConnection();

            myURLConnection.setRequestProperty("Authorization", "Token "+sessionToken);
            myURLConnection.setRequestProperty("Accept-Charset", "UTF-8");
            myURLConnection.setRequestMethod("GET");

            //This works both with compressed and non compressed files
            try( DataInputStream readerIS = new DataInputStream( myURLConnection.getInputStream())) {
                Files.copy (readerIS, Paths.get (filename));
            }
            System.out.println("Saved data in file on hard disk.");

        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }


    /**
     * Extract a file streamed from AWS (Amazon Web Services Cloud), save it to local disk
     * Using AWS results in faster downloads
     * 
     * @param filename
     * @param jobId
     */
    public void awsRetrieveDataAndSaveToFile( String filename, String jobId) {

        System.out.println("\nRetrieve compressed data directly from the AWS server, and save to hard disk in file:\n "+ filename);

        String urlGet = urlHost + "/Extractions/RawExtractionResults('"+ jobId + "')/$value";
        HttpGet requestGet = new HttpGet(urlGet);
        requestGet.addHeader("Authorization", "Token "+sessionToken);
        requestGet.addHeader("Prefer", "respond-async");
        // The next header asks for the download to occur through AWS:
        requestGet.addHeader("X-Direct-Download", "true");

        try {
            HttpClientContext context = HttpClientContext.create();
            HttpResponse responseGet = httpclient.execute(requestGet, context);

            //Handle the redirection:
            //Initialise awsURI to the initial URI (i.e. the same value as that of urlGet):
            URI awsURI = requestGet.getURI();
            //Retrieve the last Location, which is the AWS URL:
            List<URI> locations = context.getRedirectLocations();
            if (locations != null) {
                awsURI = locations.get(locations.size() - 1);
                System.out.println("\nAWS URI:\n" + awsURI);
            } else {
                throw new IOException("ERROR: could not retrieve the AWS URI");
            }

            URL myURL = awsURI.toURL();

            //To request the file from AWS we do not use the session Token (which is for the RTH server).
            //Authentication is done through the AWS URL, which is a self signed URL.
            HttpURLConnection myURLConnection = (HttpURLConnection)myURL.openConnection();

            //This works both with compressed and non compressed files
            try( DataInputStream readerIS = new DataInputStream( myURLConnection.getInputStream())) {
                Files.copy (readerIS, Paths.get (filename));
            }
            System.out.println("Saved data in file on hard disk.");

        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }


    /**
     * Read a compressed file from local disk, decompress contents and display
     * 
     * @param filename
     */
    public void readFileAndDecompress( String filename) {

    	System.out.println("\nRead compressed file from hard disk, and decompress contents.");

        try (
            InputStream theFile = new FileInputStream(filename);
            GZIPInputStream gis = new GZIPInputStream(theFile);
        ) {
            StringBuffer result = new StringBuffer();
            String line = "";
            long lineCount = 0;
            BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(gis, "UTF-8"));
            while ((line = bufferedReader.readLine()) != null) {
                ++lineCount;
                line = line + "\n";
                result.append(line);
            }
            System.out.println("\nGzipped File contents:\n"+result+"\nLine count: "+lineCount);

        } catch (IOException e) {
            e.printStackTrace();
        }
    }


    /**
     * Extract a compressed file streamed from the server, decompress it, and simply display its contents
     *  
     * @param jobId
     */
    public void retrieveDataDecompressAndDisplayContents(String jobId) {

        System.out.println("\nRetrieve compressed data directly from the RTH server, and decompress contents on the fly.");

        String urlGet = urlHost + "/Extractions/RawExtractionResults('"+ jobId + "')/$value";

        try {
            URL myURL = new URL(urlGet);

            HttpURLConnection myURLConnection = (HttpURLConnection)myURL.openConnection();

            myURLConnection.setRequestProperty("Authorization", "Token "+sessionToken);
            myURLConnection.setRequestProperty("Accept-Charset", "UTF-8");

            try (
                GzipCompressorInputStream gis = new GzipCompressorInputStream( myURLConnection.getInputStream(), true);
                BufferedReader rd = new BufferedReader(new InputStreamReader(gis, "UTF-8"));
            ) {
                StringBuffer result = new StringBuffer();
                String line = "";
                long lineCount = 0;
                while ((line = rd.readLine()) != null) {
                    ++lineCount;
                    line = line + "\n";
                    result.append(line);
                }
                System.out.println("\nData contents:\n"+result+"\nLine count: "+lineCount);
            }

        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }


    /**
     * Extract a compressed file streamed from AWS (Amazon Web Services Cloud), decompress it, and simply display its contents
     * Using AWS results in faster downloads
     *  
     * @param jobId
     */
    public void awsRetrieveDataDecompressAndDisplayContents(String jobId) {

        System.out.println("\nRetrieve compressed data directly from the AWS server, and decompress contents on the fly.");

        String urlGet = urlHost + "/Extractions/RawExtractionResults('"+ jobId + "')/$value";

        HttpGet requestGet = new HttpGet(urlGet);
        requestGet.addHeader("Authorization", "Token "+sessionToken);
        requestGet.addHeader("Prefer", "respond-async");
        // The next header asks for the download to occur through AWS:
        requestGet.addHeader("X-Direct-Download", "true");

        try {
            HttpClientContext context = HttpClientContext.create();
            HttpResponse responseGet = httpclient.execute(requestGet, context);

            //Handle the redirection:
            //Initialise awsURI to the initial URI (i.e. the same value as that of urlGet):
            URI awsURI = requestGet.getURI();
            //Retrieve the last Location, which is the AWS URL:
            List<URI> locations = context.getRedirectLocations();
            if (locations != null) {
                awsURI = locations.get(locations.size() - 1);
                System.out.println("\nAWS URI:\n" + awsURI);
            } else {
                throw new IOException("ERROR: could not retrieve the AWS URI");
            }

            URL myURL = awsURI.toURL();

            //To request the file from AWS we do not use the session Token (which is for the RTH server).
            //Authentication is done through the AWS URL, which is a self signed URL.
            HttpURLConnection myURLConnection = (HttpURLConnection)myURL.openConnection();

            try (
                GzipCompressorInputStream gis = new GzipCompressorInputStream( myURLConnection.getInputStream(), true);
                BufferedReader rd = new BufferedReader(new InputStreamReader(gis, "UTF-8"));
            ) {
                StringBuffer result = new StringBuffer();
                String line = "";
                long lineCount = 0;
                while ((line = rd.readLine()) != null) {
                    ++lineCount;
                    line = line + "\n";
                    result.append(line);
                }
                System.out.println("\nData contents:\n"+result+"\nLine count: "+lineCount);
            }

        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }


    public static void wait(int seconds) {
        try {
            System.out.println("Waiting " + seconds + " seconds ..");
            Thread.sleep(seconds * 1000);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }        
    }


    public static void promptEnterKey(){
        System.out.println("Press \"ENTER\" to continue...");
        Scanner scanner = new Scanner(System.in);
        scanner.nextLine();
    }    
} //EOF
