package com.refinitiv.dss.api.example;
/*
 * ====================================================================
 * This Java example demonstrate how to make an HTTPS request to DSS2:
 * - to create a Report Template (in this case a RTH TimeAndSales (Tick) template),
 * - to get a Report Template by Id
 * - to retrieve a Report Template by the Report Template name
 * - to get all Report Templates
 * - to delete a Report Template by Id
 * 
 * To run this example, you must provide a DSS2 username and password.
 * ====================================================================
 */

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.StringWriter;
import java.io.UnsupportedEncodingException;
import java.util.Scanner;

import org.apache.http.HttpResponse;
import org.apache.http.client.methods.HttpDelete;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.impl.client.BasicResponseHandler;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.ResponseHandler;
import org.json.JSONArray;
import org.json.JSONObject;
import org.json.JSONOrderedObject;

public class DSSReportTemplatesRTH {
    private String urlHost = "https://hosted.datascopeapi.reuters.com/RestApi/v1";
    private String sessionToken = "";
    private CloseableHttpClient httpclient = HttpClientBuilder.create().disableContentCompression().build();

    public static void main(String[] args) throws Exception {

        if(args.length < 2) {
            System.out.println("Please enter DSS2 Username as 1st command line parameter, Password as 2nd");
            System.exit(-1);
        }

        DSSReportTemplatesRTH dss2 = new DSSReportTemplatesRTH();
        dss2.getSessionToken(args[0], args[1]);
        promptEnterKey();

        // Create a TimeAndSales (Tick) Report Template
        String reportTemplateId = dss2.createRthTimeAndSalesReportTemplate("Demo_TS_Template");
        promptEnterKey();

        //* Retrieve Report Template ID from name (for the case where it was created manually)
        String myReportTemplateId = dss2.getReportTemplateId("Demo_TS_Template");
        System.out.println("Report Template ID received: " +myReportTemplateId);
        promptEnterKey();

        // Retrieve Report Template by providing an Id
        dss2.getReportTemplate(reportTemplateId);
        promptEnterKey();

        // Delete Report Template by providing an Id
        dss2.deleteReportTemplate(reportTemplateId);
        promptEnterKey();

        dss2.getAllReportTemplates();

        dss2.httpclient.close();
    }


    /**
     * Request DSS2 for a Session Token (24 hour life)
     * 
     * @param username
     * @param password
     */
    public void getSessionToken(String username, String password) {

        try {
            HttpPost httppost = new HttpPost(urlHost + "/Authentication/RequestToken");

            httppost.addHeader("content-type", "application/json; charset=UTF-8");

            JSONObject TokenRequest = new JSONObject()
                .put("Credentials", new JSONObject()
                    .put("Username",  username)
                    .put("Password",  password));

            StringEntity requestBody = new StringEntity(TokenRequest.toString());

            httppost.setEntity(requestBody);

            ResponseHandler<String> responseHandler = new BasicResponseHandler();

            String response = httpclient.execute(httppost, responseHandler);
            JSONObject jsonResponse = new JSONObject(response);

            sessionToken = jsonResponse.get("value").toString();
            System.out.println("Session Token (expires in 24 hours):\n" + sessionToken);

        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }


    /**
     * Create a RTH TimeAndSales (Tick) Report Template
     * 
     * @param reportTemplateName
     * @return reportTemplateId
     */
    public String createRthTimeAndSalesReportTemplate(String reportTemplateName) {

        String reportTemplateId = "";

        try {
            HttpPost httppost = new HttpPost(urlHost + "/Extractions/TickHistoryTimeAndSalesReportTemplates");

            httppost.addHeader("content-type", "application/json;odata.metadata=minimal");
            httppost.addHeader("Authorization", "Token " + sessionToken);

             // Note: content fields are specific to each report template type
            JSONOrderedObject reportTemplateJSONObject = new JSONOrderedObject()           
                .put("@odata.type", "#ThomsonReuters.Dss.Api.Extractions.ReportTemplates.TickHistoryTimeAndSalesReportTemplate")
                .put("ShowColumnHeaders", false)
                .put("Name", reportTemplateName)
                .put("Headers", new JSONArray() )
                .put("Trailers", new JSONArray() )
                .put("ContentFields", new JSONArray()
                    .put( new JSONObject()
                        .put("FieldName", "Quote - Ask Price")
                        .put( "Format", JSONObject.NULL))
                    .put( new JSONObject()
                        .put("FieldName", "Quote - Ask Size")
                        .put( "Format", JSONObject.NULL))
                    .put( new JSONObject()
                        .put("FieldName", "Quote - Bid Price")
                        .put( "Format", JSONObject.NULL))
                    .put( new JSONObject()
                        .put("FieldName", "Quote - Bid Size")
                        .put( "Format", JSONObject.NULL))
                    .put( new JSONObject()
                        .put("FieldName", "Quote - Exchange Time")
                        .put( "Format", JSONObject.NULL)))
                .put("Condition", new JSONOrderedObject()
                	.put("MessageTimeStampIn", "GmtUtc")
                	.put("ApplyCorrectionsAndCancellations", false)
                	.put("ReportDateRangeType", "Range")
                	.put("QueryStartDate", "2016-09-29T15:15:00.000Z")
                	.put("QueryEndDate", "2016-09-29T15:45:00.000Z"));

            System.out.println("Create report template JSON request content:\n"+ reportTemplateJSONObject.toString());
            StringEntity requestBody = new StringEntity(reportTemplateJSONObject.toString());

            httppost.setEntity(requestBody);

            ResponseHandler<String> responseHandler = new BasicResponseHandler();

            String response = httpclient.execute(httppost, responseHandler);
            JSONObject jsonResponse = new JSONObject(response);

            StringWriter out = new StringWriter();
            jsonResponse.write(out);
            System.out.println("JSON response contents:\n"+ out);
            reportTemplateId = jsonResponse.getString("ReportTemplateId");

        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }

        return reportTemplateId;
    }


    /**
    * Get Report Template ID from name
    * 
    * @param reportTemplateName
    * @return reportTemplateId
    */
    public String getReportTemplateId(String reportTemplateName) {
    	String myReportTemplateId = "";
        try {
            String urlGet = urlHost + "/Extractions/ReportTemplateGetByName(Name='" + reportTemplateName + "')";
            HttpGet request = new HttpGet(urlGet);
            request.addHeader("Authorization", "Token "+sessionToken);

            System.out.println("Retrieve Report Template with this name: "+ reportTemplateName);
            HttpResponse response = httpclient.execute(request);

            // It will receive response code 404 if no template is found for that name.
            int httpStatus = response.getStatusLine().getStatusCode();
            System.out.println("HTTP status: " + httpStatus);

            BufferedReader rd = new BufferedReader(
                new InputStreamReader(response.getEntity().getContent()));

            StringBuffer result = new StringBuffer();
            String line = "";
            while ((line = rd.readLine()) != null) {
                result.append(line);
            }

            System.out.println("JSON response contents:\n"+result);

            JSONObject jsonGetResponse = new JSONObject(result.toString());

            if (httpStatus == 200) { myReportTemplateId = jsonGetResponse.get("ReportTemplateId").toString(); };

        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }

        return myReportTemplateId;
    }


    /**
     * Get Report Template for this reportTemplateId
     * 
     * @param reportTemplateId
     */
    public void getReportTemplate(String reportTemplateId) {
        try {
            String urlGet = urlHost + "/Extractions/ReportTemplates('"+reportTemplateId+"')";
            HttpGet request = new HttpGet(urlGet);
            request.addHeader("Authorization", "Token "+sessionToken);

            System.out.println("Retrieve Report Template with this Id: "+ reportTemplateId);
            HttpResponse response = httpclient.execute(request);

            // It will receive response code 404 if no template is found for that id.
            System.out.println("HTTP status: " 
                + response.getStatusLine().getStatusCode());

            BufferedReader rd = new BufferedReader(
                new InputStreamReader(response.getEntity().getContent()));

            StringBuffer result = new StringBuffer();
            String line = "";
            while ((line = rd.readLine()) != null) {
                result.append(line);
            }

            System.out.println("Report Template received:\n"+result);

        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }


    /**
     * Get all Report Templates
     * 
     */
    public void getAllReportTemplates() {
        try {
            String urlGet = urlHost + "/Extractions/ReportTemplates";
            HttpGet request = new HttpGet(urlGet);
            request.addHeader("Authorization", "Token "+sessionToken);

            HttpResponse response = httpclient.execute(request);

            System.out.println("Get all report templates. HTTP status: " 
                + response.getStatusLine().getStatusCode());

            BufferedReader rd = new BufferedReader(
                new InputStreamReader(response.getEntity().getContent()));

            StringBuffer result = new StringBuffer();
            String line = "";
            while ((line = rd.readLine()) != null) {
                result.append(line);
            }

            System.out.println("Report Templates received:\n"+result);

            JSONObject jsonGetResponse = new JSONObject(result.toString());

            String userDetail = jsonGetResponse.get("value").toString(); 
            System.out.println("All Report Templates:\n" +userDetail);

        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }


    /**
     * Delete report template with this reportTemplateId
     * 
     * @param reportTemplateId
     */
    public void deleteReportTemplate(String reportTemplateId) {
        try {

            String urlDel = urlHost + "/Extractions/ReportTemplates('"+reportTemplateId+"')";

            HttpDelete request = new HttpDelete(urlDel);
            request.addHeader("Authorization", "Token "+sessionToken);

            System.out.println("Delete Report Template with this id: "+ reportTemplateId);
            HttpResponse response = httpclient.execute(request);

            System.out.println("HTTP status: " 
                + response.getStatusLine().getStatusCode());

            // It will receive response code 204 with no content
            if (response.getStatusLine().getStatusCode() == 204){
                System.out.println("Delete Report Template ID " + reportTemplateId + " completed.");
            }

        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }


    public static void promptEnterKey(){
        System.out.println("Press \"ENTER\" to continue...");
        Scanner scanner = new Scanner(System.in);
        scanner.nextLine();
    }    
} //EOF
