package com.refinitiv.dss.api.example;
/*
 * ====================================================================
 * This Java example demonstrates how to make an HTTPS request to DSS2
 * to search for an active Equity RIC.
 * In this example we search for active instruments, using a list of
 * exchange codes and a ticker.
 * Other combinations are of course possible.
 * See the REST API Reference Tree for more information on the search criteria.
 * 
 * Note: this search capability only returns instruments that are currently
 * active, it will not include historical instruments.
 * 
 * To run this example, you must provide a DSS2 username and password.
 * ====================================================================  
 */

import java.io.IOException;
import java.io.StringWriter;
import java.io.UnsupportedEncodingException;
import java.util.Scanner;

import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.impl.client.BasicResponseHandler;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.ResponseHandler;
import org.json.JSONArray;
import org.json.JSONOrderedObject;
import org.json.JSONObject;

public class DSSSearchEquity {
    private String urlHost = "https://hosted.datascopeapi.reuters.com/RestApi/v1";
    private String sessionToken = "";
    private CloseableHttpClient httpclient = HttpClientBuilder.create().disableContentCompression().build();

    public static void main(String[] args) throws Exception {

        if(args.length < 2) {
            System.out.println("Please enter DSS2 Username as 1st command line parameter, Password as 2nd");
            System.exit(-1);
        }

        DSSSearchEquity dss2 = new DSSSearchEquity();
        dss2.getSessionToken(args[0], args[1]);
        System.out.println("");

        // Equity search request 
        dss2.searchEquityRequest();

        dss2.httpclient.close();
    }

    
    /**
     * Request DSS2 for a Session Token (24 hour life)
     * 
     * @param username
     * @param password
     */
    public void getSessionToken(String username, String password) {

        try {
            HttpPost httppost = new HttpPost(urlHost + "/Authentication/RequestToken");

            httppost.addHeader("content-type", "application/json; charset=UTF-8");

            JSONObject TokenRequest = new JSONObject()
                .put("Credentials", new JSONObject()
                    .put("Username",  username)
                    .put("Password",  password));

            StringEntity requestBody = new StringEntity(TokenRequest.toString());

            httppost.setEntity(requestBody);

            ResponseHandler<String> responseHandler = new BasicResponseHandler();

            String response = httpclient.execute(httppost, responseHandler);
            JSONObject jsonResponse = new JSONObject(response);

            sessionToken = jsonResponse.get("value").toString();
            System.out.println("Session Token (expires in 24 hours):\n" + sessionToken);

        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }


    public void searchEquityRequest() {
        
        try {
            HttpPost httppost = new HttpPost(urlHost + "/Search/EquitySearch"); 
            
            httppost.addHeader("content-type", "application/json;odata.metadata=minimal");
            httppost.addHeader("Authorization", "Token " + sessionToken);
            httppost.addHeader("Prefer", "respond-async");
            
            JSONOrderedObject searchExtractJSONObject = new JSONOrderedObject()
                .put("SearchRequest", new JSONOrderedObject()
                    .put("AssetStatus", "Active")
                    //.put("AssetCategoryCodes", new JSONArray()  //supported values are: CEF, CMF, CIX, CON, CNI, CPR, CPI, CIN, DIB, DRC, DIS, EFU, ENI, EQU, ECL, EQI, EMS, EIF, EIO, EIW, FRX, FIN, FMS, FRB, FLT, FNI, GIN, HDF, ILB, ILF, INF, IMB, MCB, OPF, ORD, PAR, PNF, PFA, PRF, RTS, TIN, UNC, UNT, YCT, ZCB
                    //    .put("ORD"))  //Currently only supports a single value
                    //.put("SubTypeCodes", new JSONArray() //supported values are: CVPR, DPRC, UNKN, FIDX, HYBD, OPFN, ODSH, PART, PFSH, RGHT, UNIT, WRNT                        
                    //    .put("ODSH"))  //Currently only supports a single value
                    //.put("CurrencyCodes", new JSONArray()
                    //    .put("EUR"))  //Currently only supports a single value 
                    //.put("CompanyName", "Peugeot")
                    //.put("Description", "PEUGEOT")
                    //.put("DomicileCodes", new JSONArray()
                    //    .put("FR"))  //Currently only supports a single value
                    .put("ExchangeCodes", new JSONArray()
                        .put("PAR")
                        .put("SWX"))
                    //.put("FairValueIndicator", "Yes")
                    //.put("FileCodes", new JSONArray()
                    //    .put("1305")
                    //    .put("2032"))
                    //.put("GicsCodes", new JSONArray()
                    //    .put("25102010")
                    //    .put("25102020"))
                    //.put("OrgId", "18175")
                    .put("Ticker", "UG")
                    //.put("IdentifierType", "Ric")
                    //.put("PreferredIdentifierType", "Ric")
                );

            System.out.println("Equity search JSON request content:\n"+ searchExtractJSONObject.toString());
            StringEntity requestBody = new StringEntity(searchExtractJSONObject.toString());

            httppost.setEntity(requestBody);

            ResponseHandler<String> responseHandler = new BasicResponseHandler();
            
            String response = httpclient.execute(httppost, responseHandler);
            JSONObject jsonResponse = new JSONObject(response);

            StringWriter out = new StringWriter();
            jsonResponse.write(out);
            System.out.println("\nJSON response contents:\n"+ out);

        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

 
    public static void promptEnterKey() {
        System.out.println("Press \"ENTER\" to continue...");
        Scanner scanner = new Scanner(System.in);
        scanner.nextLine();
    }
} //EOF
