package com.refinitiv.dss.api.example;
/*
 * ====================================================================
 * This Java example demonstrates how to make an HTTPS request to DSS2
 * to search for an active Future or Option RIC.
 * In this example we search for instruments, using 2 main criteria:
 * - strike price within a specific range, and 
 * - expiration after a specified date
 * Other combinations are of course possible.
 * See the REST API Reference Tree for more information on the search criteria.
 * 
 * Note: this search capability only returns instruments that are currently
 * active, it will not include historical instruments.
 * 
 * To run this example, you must provide a DSS2 username and password.
 * ====================================================================  
 */

import java.io.IOException;
import java.io.StringWriter;
import java.io.UnsupportedEncodingException;
import java.util.Scanner;

import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.impl.client.BasicResponseHandler;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.ResponseHandler;
import org.json.JSONOrderedObject;
import org.json.JSONObject;

public class DSSSearchFuture {
    private String urlHost = "https://hosted.datascopeapi.reuters.com/RestApi/v1";
    private String sessionToken = "";
    private CloseableHttpClient httpclient = HttpClientBuilder.create().disableContentCompression().build();

    public static void main(String[] args) throws Exception {

        if(args.length < 2) {
            System.out.println("Please enter DSS2 Username as 1st command line parameter, Password as 2nd");
            System.exit(-1);
        }

        DSSSearchFuture dss2 = new DSSSearchFuture();
        dss2.getSessionToken(args[0], args[1]);
        System.out.println("");

        // Future search request 
        dss2.searchFutureRequest();

        dss2.httpclient.close();
    }

    
    /**
     * Request DSS2 for a Session Token (24 hour life)
     * 
     * @param username
     * @param password
     */
    public void getSessionToken(String username, String password) {

        try {
            HttpPost httppost = new HttpPost(urlHost + "/Authentication/RequestToken");

            httppost.addHeader("content-type", "application/json; charset=UTF-8");

            JSONObject TokenRequest = new JSONObject()
                .put("Credentials", new JSONObject()
                    .put("Username",  username)
                    .put("Password",  password));

            StringEntity requestBody = new StringEntity(TokenRequest.toString());

            httppost.setEntity(requestBody);

            ResponseHandler<String> responseHandler = new BasicResponseHandler();

            String response = httpclient.execute(httppost, responseHandler);
            JSONObject jsonResponse = new JSONObject(response);

            sessionToken = jsonResponse.get("value").toString();
            System.out.println("Session Token (expires in 24 hours):\n" + sessionToken);

        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    
    public void searchFutureRequest() {
        
        try {
            HttpPost httppost = new HttpPost(urlHost + "/Search/FuturesAndOptionsSearch"); 
            
            httppost.addHeader("content-type", "application/json;odata.metadata=minimal");
            httppost.addHeader("Authorization", "Token " + sessionToken);
            httppost.addHeader("Prefer", "respond-async");
            
            JSONOrderedObject searchExtractJSONObject = new JSONOrderedObject()
                .put("SearchRequest", new JSONOrderedObject()
                    //.put("FileCodes", JSONObject.NULL)
                    //.put("FileCodes", new JSONArray()
                    //    .put("5054")
                    //    .put("5451"))
                    //.put("CurrencyCodes", new JSONArray()
                    //    .put("USD"))  //Currently only supports a single value 
                    //.put("ExchangeCodes", new JSONArray()
                    //    .put("OPQ")
                    //    .put("PAR"))
                    .put("StrikePrice", new JSONOrderedObject()
                        .put("@odata.type", "#ThomsonReuters.Dss.Api.Search.NumericRangeComparison")
                        .put("From", 85)
                        .put("To", 88))
                    .put("ExpirationDate", new JSONOrderedObject()
                        .put("@odata.type", "#ThomsonReuters.Dss.Api.Search.DateValueComparison")
                        .put("ComparisonOperator", "GreaterThanEquals")
                        .put("Value", "2016-12-31T00:00:00.000Z"))
                    .put("IdentifierType", "Ric")
                    .put("Identifier", "ED*")
                    .put("PreferredIdentifierType", "Ric")
                    .put("UnderlyingRic", "ED")
                );

            System.out.println("Future search JSON request content:\n"+ searchExtractJSONObject.toString());
            StringEntity requestBody = new StringEntity(searchExtractJSONObject.toString());

            httppost.setEntity(requestBody);

            ResponseHandler<String> responseHandler = new BasicResponseHandler();
            
            String response = httpclient.execute(httppost, responseHandler);
            JSONObject jsonResponse = new JSONObject(response);

            StringWriter out = new StringWriter();
            jsonResponse.write(out);
            System.out.println("\nJSON response contents:\n"+ out);

        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }


    public static void promptEnterKey() {
        System.out.println("Press \"ENTER\" to continue...");
        Scanner scanner = new Scanner(System.in);
        scanner.nextLine();
    }
} //EOF
