package com.refinitiv.dss.api.example;
/*
 * ====================================================================
 * This Java example demonstrate how to make an HTTPS request to DSS2 to:
 * - retrieve a Session Token,
 * - get current user information, 
 * - get current user preferences,
 * - get user preferences for a given user.
 * 
 * To run this example, you must provide a DSS2 username and password.
 * ====================================================================
 */

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.net.MalformedURLException;
import java.util.Scanner;

import org.apache.http.HttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.impl.client.BasicResponseHandler;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.ResponseHandler;
import org.json.JSONException;
import org.json.JSONObject;

public class DSSUserInfo {
    private String urlHost = "https://hosted.datascopeapi.reuters.com/RestApi/v1";
    private String sessionToken = "";
    private CloseableHttpClient httpclient = HttpClientBuilder.create().build();

    public static void main(String[] args) throws Exception {

        if(args.length < 2) {
            System.out.println("Please enter DSS2 Username as 1st command line parameter, Password as 2nd");
            System.exit(-1);
        }

        DSSUserInfo dss2 = new DSSUserInfo();
        dss2.getSessionToken(args[0], args[1]);
        promptEnterKey();
        // GET current user
        dss2.getCurrentUser();
        promptEnterKey();
        dss2.getCurrentUserPreference();
        promptEnterKey();
        dss2.getUserPreference(args[0]);

        dss2.httpclient.close();
    }


    /**
     * Request DSS2 for a Session Token (24 hour life)
     * 
     * @param username
     * @param password
     */
    public void getSessionToken(String username, String password) {

        try {
            HttpPost httppost = new HttpPost(urlHost + "/Authentication/RequestToken");

            httppost.addHeader("content-type", "application/json; charset=UTF-8");

            JSONObject TokenRequest = new JSONObject()
                .put("Credentials", new JSONObject()
                    .put("Username",  username)
                    .put("Password",  password));

            StringEntity requestBody = new StringEntity(TokenRequest.toString());

            httppost.setEntity(requestBody);

            ResponseHandler<String> responseHandler = new BasicResponseHandler();

            String response = httpclient.execute(httppost, responseHandler);
            JSONObject jsonResponse = new JSONObject(response);

            sessionToken = jsonResponse.get("value").toString();
            System.out.println("Session Token (expires in 24 hours):\n" + sessionToken);

        } catch (MalformedURLException e) {
            e.printStackTrace();
        } catch (JSONException e) {
            e.printStackTrace();
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }


    /**
     * Get Current User detail from DSS2
     * 
     */
    public void getCurrentUser() {
        try {
            String urlGet = urlHost + "/Users/Users";

            HttpGet request = new HttpGet(urlGet);
            request.addHeader("Authorization", "Token "+sessionToken);

            HttpResponse response = httpclient.execute(request);

            System.out.println("HTTP status: " 
                + response.getStatusLine().getStatusCode());

            BufferedReader rd = new BufferedReader(
                new InputStreamReader(response.getEntity().getContent()));

            StringBuffer result = new StringBuffer();
            String line = "";
            while ((line = rd.readLine()) != null) {
                result.append(line);
            }

            System.out.println("Results received: "+result);

            JSONObject jsonGetResponse = new JSONObject(result.toString());
            String userDetail = jsonGetResponse.get("value").toString(); 
            System.out.println("Current User Details: " +userDetail);

        } catch (MalformedURLException e) {
            e.printStackTrace();
        } catch (JSONException e) {
            e.printStackTrace();
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }


    /**
     * Get Current User Preference
     * 
     */
    public void getCurrentUserPreference() {
        try {
            String urlGet = urlHost + "/Users/UserPreferences";

            HttpGet request = new HttpGet(urlGet);
            request.addHeader("Authorization", "Token "+sessionToken);

            HttpResponse response = httpclient.execute(request);

            System.out.println("HTTP status: " 
                + response.getStatusLine().getStatusCode());

            BufferedReader rd = new BufferedReader(
                new InputStreamReader(response.getEntity().getContent()));

            StringBuffer result = new StringBuffer();
            String line = "";
            while ((line = rd.readLine()) != null) {
                result.append(line);
            }

            System.out.println("Results received: "+result);

            JSONObject jsonGetResponse = new JSONObject(result.toString());

            String userDetail = jsonGetResponse.get("value").toString(); 
            System.out.println("Current User Preferences: " +userDetail);

        } catch (MalformedURLException e) {
            e.printStackTrace();
        } catch (JSONException e) {
            e.printStackTrace();
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    
    /**
     * Get User Preference for a specific User
     * 
     * @param username
     */
    public void getUserPreference(String username) {
        try {
            String urlGet = urlHost +"/Users/Users("+username+")/Preferences";

            HttpGet request = new HttpGet(urlGet);
            request.addHeader("Authorization", "Token "+sessionToken);

            HttpResponse response = httpclient.execute(request);

            System.out.println("HTTP status: " 
                + response.getStatusLine().getStatusCode());

            BufferedReader rd = new BufferedReader(
                new InputStreamReader(response.getEntity().getContent()));

            StringBuffer result = new StringBuffer();
            String line = "";
            while ((line = rd.readLine()) != null) {
                result.append(line);
            }

            System.out.println("Results received: "+result);

            JSONObject jsonGetResponse = new JSONObject(result.toString());
            String userSettings = jsonGetResponse.get("ContentSettings").toString(); 
            String uiSettings = jsonGetResponse.get("UiSettings").toString(); 
            System.out.println("User ContentSettings: " +userSettings);
            System.out.println("User UI settings: " +uiSettings);

        } catch (MalformedURLException e) {
            e.printStackTrace();
        } catch (JSONException e) {
            e.printStackTrace();
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

 
    public static void promptEnterKey(){
        System.out.println("Press \"ENTER\" to continue...");
        Scanner scanner = new Scanner(System.in);
        scanner.nextLine();
    }
} //EOF
